'use strict';

Object.defineProperty(exports, "__esModule", {
			value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = reducer;
exports.initial_form_state = initial_form_state;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function reducer() {
			var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
			var action = arguments[1];

			// Temporary debugging
			// console.log('Action', action)
			// console.log('Previous state', state)

			var form_state = state[action.form];

			switch (action.type) {
						case '@@simpler-redux-form/initialize':

									// It will have been already initialized on the server-side
									// if (state[action.form] !== undefined)
									// {
									// 	throw new Error(`Form "${action.form}" has already been initialized.`)
									// }

									state = _extends({}, state, _defineProperty({}, action.form, initial_form_state(action.values)));

									return state;

						case '@@simpler-redux-form/destroy':

									// if (state[action.form] === undefined)
									// {
									// 	throw new Error(`Form "${action.form}" has already been destroyed.`)
									// }

									state = _extends({}, state);

									delete state[action.form];

									return state;

						case '@@simpler-redux-form/register-field':

									state = _extends({}, state);

									// Uses a numerical counter instead of a boolean.
									// https://github.com/erikras/redux-form/issues/1705
									// If the value is `0` then it means that the field
									// has been previously initialized so not reinitializing it.
									// This also preserves the initial value of the field.
									if (form_state.fields[action.field] === undefined) {
												form_state.fields[action.field] = 1;

												var field_value = action.value !== undefined ? action.value : form_state.initial_values[action.field];
												var field_error = action.validate(field_value);

												// Only initializes the field with its default value
												// if it hasn't been seen before.
												// Otherwise will initialize the field with its current value.
												form_state.values[action.field] = field_value;
												form_state.errors[action.field] = field_error;

												form_state.initial_values[action.field] = field_value;
												form_state.initial_value_errors[action.field] = field_error;

												// If an external error was specified, then show it.
												if (action.non_validation_error) {
															form_state.indicate_invalid[action.field] = true;
												} else {
															form_state.indicate_invalid[action.field] = false;
												}
									} else {
												form_state.fields[action.field]++;
									}

									return state;

						case '@@simpler-redux-form/unregister-field':

									// Seems that a form gets destroyed before its fields,
									// so `form_state` could be undefined (probably).
									if (form_state) {
												state = _extends({}, state);

												if (!form_state.fields[action.field]) {
															console.error('Warning: An "unregister field" request was sent for field "' + action.field + '" which is not currently registered. This is a bug and it needs to be reported: https://github.com/catamphetamine/simpler-redux-form/issues');
												}

												// Uses a numerical counter instead of a boolean.
												// https://github.com/erikras/redux-form/issues/1705
												form_state.fields[action.field]--;

												// Even if the registration counter for a field
												// becomes equal to `0` it's still not destroyed,
												// because theoretically it could correspond to a new field
												// being added in the beginning of the field list
												// therefore causing all field to unregister and then register again.
												// If those fields were destroyed then their values would be lost.
												// In almost all cases the decision to not delete the field info is not a memory leak
												// because all fields finally get destroyed when the form is unmounted
												// and also almost all forms don't ever unmount fields.
									}

									return state;

						case '@@simpler-redux-form/changed':

									// This changes [action.form] object's properties,
									// and therefore Redux'es shallow compare
									// rerenders the form every time.
									//
									// state =
									// {
									// 	...state,
									// 	[action.form]:
									// 	{
									// 		...form_state,
									// 		values:
									// 		{
									// 			...form_state.values,
									// 			[action.field] : action.value
									// 		},
									// 		errors:
									// 		{
									// 			...form_state.errors,
									// 			[action.field] : action.error
									// 		},
									// 		indicate_invalid:
									// 		{
									// 			...form_state.indicate_invalid,
									// 			[action.field] : false
									// 		}
									// 	}
									// }

									state = _extends({}, state);

									form_state.values[action.field] = action.value;
									form_state.errors[action.field] = action.error;
									form_state.indicate_invalid[action.field] = false;

									return state;

						case '@@simpler-redux-form/indicate-invalid':

									// This changes [action.form] object's properties,
									// and therefore Redux'es shallow compare
									// rerenders the form every time.
									//
									// state =
									// {
									// 	...state,
									// 	[action.form]:
									// 	{
									// 		...form_state,
									// 		indicate_invalid:
									// 		{
									// 			...form_state.indicate_invalid,
									// 			[action.field] : true
									// 		}
									// 	}
									// }

									state = _extends({}, state);

									form_state.indicate_invalid[action.field] = true;

									return state;

						case '@@simpler-redux-form/dont-indicate-invalid':

									state = _extends({}, state);

									form_state.indicate_invalid[action.field] = false;

									return state;

						case '@@simpler-redux-form/set':

									state = _extends({}, state);

									form_state.values[action.field] = action.value;
									form_state.errors[action.field] = action.error;
									form_state.indicate_invalid[action.field] = action.error ? true : false;

									return state;

						case '@@simpler-redux-form/focus':

									state = _extends({}, state);

									for (var _iterator = Object.keys(form_state.focus), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
												var _ref;

												if (_isArray) {
															if (_i >= _iterator.length) break;
															_ref = _iterator[_i++];
												} else {
															_i = _iterator.next();
															if (_i.done) break;
															_ref = _i.value;
												}

												var field = _ref;

												delete form_state.focus[field];
									}

									form_state.focus[action.field] = true;

									return state;

						case '@@simpler-redux-form/focused':

									state = _extends({}, state);

									for (var _iterator2 = Object.keys(form_state.focus), _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
												var _ref2;

												if (_isArray2) {
															if (_i2 >= _iterator2.length) break;
															_ref2 = _iterator2[_i2++];
												} else {
															_i2 = _iterator2.next();
															if (_i2.done) break;
															_ref2 = _i2.value;
												}

												var _field = _ref2;

												delete form_state.focus[_field];
									}

									return state;

						case '@@simpler-redux-form/scroll':

									state = _extends({}, state);

									for (var _iterator3 = Object.keys(form_state.scroll_to), _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _iterator3[Symbol.iterator]();;) {
												var _ref3;

												if (_isArray3) {
															if (_i3 >= _iterator3.length) break;
															_ref3 = _iterator3[_i3++];
												} else {
															_i3 = _iterator3.next();
															if (_i3.done) break;
															_ref3 = _i3.value;
												}

												var _field2 = _ref3;

												delete form_state.scroll_to[_field2];
									}

									form_state.scroll_to[action.field] = true;

									return state;

						case '@@simpler-redux-form/scrolled':

									state = _extends({}, state);

									for (var _iterator4 = Object.keys(form_state.scroll_to), _isArray4 = Array.isArray(_iterator4), _i4 = 0, _iterator4 = _isArray4 ? _iterator4 : _iterator4[Symbol.iterator]();;) {
												var _ref4;

												if (_isArray4) {
															if (_i4 >= _iterator4.length) break;
															_ref4 = _iterator4[_i4++];
												} else {
															_i4 = _iterator4.next();
															if (_i4.done) break;
															_ref4 = _i4.value;
												}

												var _field3 = _ref4;

												delete form_state.scroll_to[_field3];
									}

									return state;

						case '@@simpler-redux-form/on-field-focused':

									state = _extends({}, state);

									form_state.misc.latest_focused_field = action.field;

									return state;

						case '@@simpler-redux-form/validation-passed':

									state = _extends({}, state);

									form_state.misc.validation_failed = !action.passed;

									return state;

						case '@@simpler-redux-form/reset-invalid-indication':

									state = _extends({}, state);

									for (var _iterator5 = Object.keys(form_state.indicate_invalid), _isArray5 = Array.isArray(_iterator5), _i5 = 0, _iterator5 = _isArray5 ? _iterator5 : _iterator5[Symbol.iterator]();;) {
												var _ref5;

												if (_isArray5) {
															if (_i5 >= _iterator5.length) break;
															_ref5 = _iterator5[_i5++];
												} else {
															_i5 = _iterator5.next();
															if (_i5.done) break;
															_ref5 = _i5.value;
												}

												var _field4 = _ref5;

												form_state.indicate_invalid[_field4] = false;
									}

									form_state.misc.validation_failed = false;

									return state;

						default:
									return state;
			}
}

function initial_form_state() {
			var initial_values = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			return {
						// Whether the form has been "initialized".
						// The form is considered "initialized"
						// when all its fields have been initialized
						// which is some time after it has been created,
						// and which is some time after the first `@connect()` is called.
						initialized: false,

						// All form `<Field/>`s
						fields: {},

						// All form `<Field/>` values
						values: {},

						// Initial form `<Field/>` values
						initial_values: initial_values,

						// `validate()` results for initial form `<Field/>` values
						initial_value_errors: {},

						// `validate()` results for current form `<Field/>` values
						errors: {},

						// Whether `validate()` errors for current form `<Field/>` values should be displayed
						indicate_invalid: {},

						// If a `<Field/>` is being focused programmatically then
						// the `focus` object contains `<Field/>` `name` set to `true`.
						focus: {},

						// If a `<Field/>` is being scrolled to programmatically then
						// the `scroll_to` object contains `<Field/>` `name` set to `true`.
						scroll_to: {},

						// Form properties which should not trigger re-`render()`ing are kept here.
						misc: {
									// Whether `validate()`s for all form `<Field/>`s pass
									validation_failed: undefined,

									// Is used for tracking abandoned forms in Google Analytics
									latest_focused_field: undefined
						}
			};
}
//# sourceMappingURL=reducer.js.map